# -*- coding: UTF-8 -*-

import os

from artix.common.dict2xml import Converter
from common import filesmodify, logger, macros

logInfo = "AIF xml converter: "
saleFiles = set()


def parseFlag(flag):
    f = open(flag, 'r')
    shifts = None
    dates = None
    if f.readline().strip() == '###':
        str = f.readline().strip()
        if len(str.split('-')) == 2:
            if str.find('.') == -1:
                shifts = str.replace('-', '@')
            else:
                fromDate = str.split('-')[0].split('.')
                toDate = str.split('-')[1].split('.')
                dates = '%s-%s-%sT00:00:00.0@%s-%s-%sT23:59:59.99' % (fromDate[2], fromDate[1], fromDate[0], toDate[2], toDate[1], toDate[0])
    f.close()
    return shifts, dates

def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "Start unload sales for cash №" + str(cash.code))
    resultPath = node.currentConverter.onlineSalePath if currentSales else node.currentConverter.path
    log.debug('currentSales=%s resultPath=%s'%(currentSales,resultPath))
    log.debug('shopcode=%s cashcode=%s'%(node.id,cash.code))
    currentPath = macros.applyMacros(resultPath, log, shopcode = node.id, cashcode = cash.code)
    log.debug('currentPath=%s'%(currentPath))
    salePath = workDirectory + '/' + os.path.basename(currentPath)
    log.debug('workDirectory=%s'%(workDirectory))
    if os.path.isfile(salePath):
        log.info("%sfile %s exists. Delete old sales" % (logInfo, salePath))

    emptyFile = True
    log.debug('node.currentConverter.dataType = %s' % node.currentConverter.dataType)
    documentsList = []
    if node.currentConverter.dataType in ['sales','all']:
        for i in salesFunc():
            documentsList.append(i)
        if documentsList:
            emptyFile = False

    securitylogsList = []
    if node.currentConverter.dataType in ['crights','all']:
        for i in securityLogsFunc():
            securitylogsList.append(i)
        if securitylogsList:
            emptyFile = False
    shiftsList = []
    if node.currentConverter.dataType in ['sales','all']:
        for i in shiftsFunc():
            if not emptyFile or (node.currentConverter.withClosedShifts and i['timeEnd']):
                shiftsList.append(i)
        if node.currentConverter.withClosedShifts and shiftsList:
            emptyFile = False

    if emptyFile:
        log.info('Sales data is empty')
    else:
        salesDict = {'documents': documentsList, 'securitylogs': securitylogsList, 'shifts': shiftsList}

        # оборачиваем все тело в <root>
        # для вложенных тегов делаем смещение 4 пробела
        # формируем пустой тег xml для полей со значениями None, [], {}
        xml = Converter(wrap ='root', indent ="    ").build(salesDict, closed_tags_for=[None, [], {}])
        del documentsList[:]
        del documentsList
        del securitylogsList[:]
        del securitylogsList
        del shiftsList[:]
        del shiftsList
        log.info("%sCreate file %s" % (logInfo, salePath))
        with open(salePath, 'w') as f:
            f.write("<?xml version='1.0'?>\n")
            f.write(xml)

        log.info("%ssales generated" % logInfo)
        statusFileName = "ready"
        statusDir = workDirectory
        if currentSales:
            statusFileName += "Online"
            statusDir = os.path.dirname(workDirectory)
        converterSalesReadyFile = open('%s/%s' % (statusDir, statusFileName), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/%s' % (os.path.dirname(os.path.dirname(statusDir)), statusFileName), 'w')
        salesReadyFile.close()
        global saleFiles
        saleFiles = set((salePath, converterSalesReadyFile.name, salesReadyFile.name))


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path),
            rewrite=node.currentConverter.rewriteDestFile, backupDir=backupDir,
            maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.onlineSalePath))


def getSaleFiles():
    global saleFiles
    return saleFiles
