# -*- coding: UTF-8 -*-
import sys
from datetime import datetime, date

"""
Маппер из сущности ukm в сущность artix
"""


class UkmToArtixTmcMapper(object):
    tnvd_mapper = dict()
    measure_mapper = dict()
    tnvds_from_xml = dict()

    subexcise_to_paymentobject_by_egaisType = {
        "0": lambda subExcise: 2 if int(subExcise) == 1 else 0,
        "1": lambda subExcise: 2,
        "3": lambda subExcise: 31 if int(subExcise) == 1 else 33,
        "4": lambda subExcise: 31 if int(subExcise) == 1 else 33
    }

    opmode_by_egaisType = {
        "0": "0",
        "1": "128",
        "3": "0",
        "4": "0"
    }

    @classmethod
    def get_tmctype(cls, tnvd_group_id, egais_type):
        """
        получение типа товара по коду тнвд из справочника и по полю egaisType
        сомнительно, но допустим
        """

        if not tnvd_group_id and int(egais_type) == 1:
            # для алкогольных товаров в справочнике не заполняется код тнвд
            return 1
        # при отключенной настройке маппинг не будет считан, тогда просто парсим egaisType
        if len(cls.tnvd_mapper) == 0:
            return 7 if int(egais_type) == 3 or int(egais_type) == 4 else egais_type
        # по id записи, указанной в товаре получаем код группы тнвд
        # из списка, считанного из xml-файла
        code = cls.tnvds_from_xml.get(tnvd_group_id, None)
        # по коду группы тнвд получаем tmctype (из маппинга, указанного в настройках)
        # маппинг вида  <tmctype>=code1,code2,code3
        if code:
            for k, v in cls.tnvd_mapper.items():
                codes = v.split(',')
                if code in codes:
                    return k
        # по умолчанию возвращаем просто признак маркированного товара
        return 7

    @classmethod
    def get_measure_code(cls, measure_name):
        """
        получение кода единицы измерения из маппинга, указанного в настройках
        <код артикс>=<название из ukm>
        """
        if measure_name is not None:
            # во избежание проблем с кириллицей на python2
            if sys.version_info.major == 2:
                measure_name = measure_name.encode("UTF-8")
            for k, v in cls.measure_mapper.items():
                if v == measure_name:
                    return k
        # по умолчанию возвращаем 1
        return 1

    @classmethod
    def get_paymentobject(cls, egaisType, subExcise):
        """
        egaisType == 0 && subExcise == 0 =>  0
        egaisType == 0 && subExcise == 1 =>  2
        egaisType == 1 && subExcise == 0 =>  2
        egaisType == 1 && subExcise == 1 =>  2
        egaisType == 3 && subExcise == 0 =>  33
        egaisType == 3 && subExcise == 1 =>  31
        egaisType == 4 && subExcise == 0 =>  33
        egaisType == 4 && subExcise == 1 =>  31
        """
        return cls.subexcise_to_paymentobject_by_egaisType.get(egaisType, lambda x: 0)(subExcise if subExcise else 0)

    @classmethod
    def get_opmode(cls, egaisType):
        """
        egaisType == 0 =>  0
        egaisType == 1 => 128
        egaisType == 3 => 0
        egaisType == 4 => 0
        """
        return cls.opmode_by_egaisType.get(egaisType, "0")

    @classmethod
    def map_tmc(cls, ukm_tmcs, tnvds=None, measure_map=None, tnvd_map=None):
        cls.measure_mapper = measure_map if measure_map is not None else {}
        cls.tnvds_from_xml = tnvds if tnvds is not None else {}
        cls.tnvd_mapper = tnvd_map if tnvd_map is not None else {}
        for ukm in ukm_tmcs:
            name = ukm["name"].replace("&amp;", "&")
            barcodes = list()
            for bcode in ukm["barcode"]:
                barcode = {'name': name, 'code': ukm["article"],
                           "cquant": 1 if bcode["quantity"] and int(bcode["quantity"]) == 0 else bcode["quantity"],
                           "barcode": bcode["id"]}
                barcodes.append(barcode)
            tmc = dict()
            tmc['name'] = name
            tmc['articul'] = ukm["article"]
            tmc['inventcode'] = ukm["article"]
            tmc['inventgroup'] = ukm.get("groupId", None)
            tmc['barcodes'] = barcodes
            tmc['tmctype'] = int(
                UkmToArtixTmcMapper.get_tmctype(ukm.get("TNVDcode", 0), ukm.get("egaisType", 0)) if int(
                    ukm.get("egaisType", 0)) >= 1 else 0)
            tmc['measurecode'] = UkmToArtixTmcMapper.get_measure_code(ukm["measure"])
            tmc['taxgroupcode'] = ukm.get("taxgroupId", None)
            tmc['paymentobject'] = UkmToArtixTmcMapper.get_paymentobject(ukm.get("egaisType", 0),
                                                                         ukm.get("SubExcise", 0))
            tmc['opmode'] = UkmToArtixTmcMapper.get_opmode(ukm.get("egaisType", 0))
            tmc["barcode"] = None
            tmc["price"] = 0.0
            yield tmc

    @classmethod
    def map_prices(cls, ukm_prices):
        """
        маппинг переоценок
        """
        for price_data in ukm_prices:
            price = dict()
            price['price'] = price_data["value"]
            price['minprice'] = price_data["minprice"]
            price['minretailprice'] = price_data["mrc"] if price_data["mrc"] else price_data["minprice"]
            price['barcode'] = price_data["article"]
            price['documentid'] = 1
            price['doctype'] = 1
            price['effectivedate'] = datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S")
            yield price

    @classmethod
    def map_tprices(cls, ukm_prices):
        """
        маппинг доп.цен
        """
        return ({'price': price["value"],
                 'pricecode': price["code"],
                 'name': price["name"],
                 'barcode': price["article"],
                 'documentid': 1,
                 'effectivedate': datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S")
                 } for price in ukm_prices)

    @classmethod
    def map_alcocode(cls, ukm_alcocodes):
        """
        маппинг алкокодов
        """
        return ({"alcocode": alcocode["AlcCode"], 'tmccode': alcocode["article"]} for alcocode in ukm_alcocodes)

    @classmethod
    def map_groups(cls, ukm_groups):
        """
        маппинг групп товаров
        """
        return ({'groupcode': group["id"],
                 "parentgroupcode": group["owner"],
                 "groupname": group["name"].replace("&amp;", "&")} for group in ukm_groups)

    @classmethod
    def map_tax(cls, ukm_taxes):
        """
        маппинг налогов
        """
        return ({'code': tax["id"],
                 'name': tax["name"],
                 'rate': str(tax["percent"]).replace('%', '')} for tax in ukm_taxes)

    @classmethod
    def map_kit(cls, ukm_kits):
        """
        маппинг комплектов товаров и сущностей внутри них
        """
        kit_items = []
        for kit in ukm_kits:
            kit_items.extend(kit["subItem"])
        kits = ({'barcode': k["article"], 'name': k["article"]} for k in ukm_kits)
        kit_items = ({'tmccode': item["article"], 'quantity': item["quantity"], 'tmckitbarcode': item["parentcode"]} for
                     item in kit_items)
        return kits, kit_items
