# -*- coding: utf-8 -*-

import sys
import os
import optparse
exchangersRootPath = "/linuxcash/cash/exchangesystems/exchangers/src"
tmpPath = ""

def setPath():
    """
    Подмена рабочей директории
    """
    global tmpPath
    tmpPath = sys.path[0]
    sys.path[0] = exchangersRootPath

def resetPath():
    """
    Возврат рабочей директории
    """
    sys.path[0] = tmpPath

setPath()
from common import config, logger, macros, registry
log = logger.getUploadLogger("main")
from upload.fam import fam
resetPath()

reg = registry.Registry()
shopCode = str(reg.shopcode())
cashCode = str(reg.cashcode())


def main(argv):
    """
    Запуск одной итерации обмена
    """
    setPath()    
    parser = optparse.OptionParser()
    parser.add_option("-v", "--converter", dest="converterName", help=u"Тип конвертора: aif, oracle, shtrih, smag-ukm, smag-ukm4, users, crystal, atol")
    parser.add_option("-f", "--flag", dest="flags", help=u"Путь к флагам")
    parser.add_option("-d", "--data", dest="dataPath", help=u"Путь к справочникам")  
    (options, args) = parser.parse_args(argv)
    
    workDirectory = config.getString('main', 'workDirectory', '/linuxcash/cash/data/tmp/exchangers')
    try:
        nodeGetter = __import__('upload.node.%s.nodeGetter' % (config.getString('UPLOAD', 'nodeGetter', 'cash')), fromlist='q')
        node = nodeGetter.getNodes()[0]
    except Exception as e:
        log.error(u'Ошибка считывания нод. %s' % str(e))        
    if options.converterName:
        if not any(x.name == options.converterName for x in node.converters):
            # Если конвертора из параметра нет в ноде, добавим его сами
            converter = nodeGetter.getConverter(options.converterName)
            if converter:
                node.converters.append(converter)
                log.info(u'Конвертер - \'%s\'' % options.converterName)
            else:
                log.error(u"Не поддерживаемый тип конвертера %s" % options.converterName)
                return
        for converter in node.converters:
            if converter.name == options.converterName and options.dataPath:
                converter.data = [ macros.applyMacros(d.strip(), log, shopcode = shopCode, cashcode = cashCode) for d in options.dataPath.split(',') ]
                log.info(u'Файлы данных - \'%s\'' % converter.data)
            if converter.name == options.converterName and options.flags:
                converter.flags = [ macros.applyMacros(fl.strip(), log, shopcode = shopCode, cashcode = cashCode) for fl in options.flags.split(',') ]       
                log.info(u'Файлы флагов - \'%s\'' % converter.flags)   
    try:        
        for converter in node.converters:
            for fl in converter.flags:
                if os.path.exists(os.path.dirname(fl)):
                    node.currentFlag = fam.findFlag(os.path.dirname(fl), os.path.basename(fl))
                    if node.currentFlag:
                        node.currentConverter = converter
                        fam.convert(workDirectory, node)
                        break
    except Exception as e:
        resetPath()
        log.error(u'error %s' % e)
    resetPath()

if __name__ == '__main__':
    main(sys.argv[1:])