# -*- coding: UTF-8 -*-
'''
Created on 29.03.2012

@author: Chaynikov Vladimir && kov
'''

import re
from common.parameters import QueueParameters


class QueueInserterConfig(object):
    name = 'queue'

    def __init__(self, queue):
        if isinstance(queue, QueueParameters):
            self.queue = queue
        else:
            raise AttributeError(u"Ошибка. Объект 'queue' не принадлежит классу 'QueueParameters'")


class DBInserterConfig(object):
    name = 'db'

    def __init__(self):
        pass


class ConverterParameters(object):
    name = 'default'

    def __init__(self, flags, data, errorDest, backup_dir):
        self.flags = flags
        self.data = data
        self.errorDest = errorDest
        self.backup_dir = backup_dir


class AifConverterParameters(ConverterParameters):
    name = 'aif'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, hotKeyByBarcode=False, checkMatchFileNames=False):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.hotKeyByBarcode = hotKeyByBarcode
        self.checkMatchFileNames = checkMatchFileNames


class OracleConverterParameters(ConverterParameters):
    name = 'oracle'

    def __init__(self, flags, data, errorDest=None, backup_dir=None):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)


class SmagConverterParameters(ConverterParameters):
    name = 'smag-ukm'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, charset='866', format='dat',
                 defaultRoleCode=3, additionallyRoleCode=2, usersWithAdditionallyRole='',
                 relationRolesBy='name', substringInAdminName='Администратор', catalogBindCode='', groupBindCode='',
                 setMinPriceIfDiscountRestrictMissing=True, group2CatalogMapper={}):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.charset = charset
        self.format = format
        self.defaultRoleCode = defaultRoleCode
        self.additionallyRoleCode = additionallyRoleCode
        self.usersWithAdditionallyRole = usersWithAdditionallyRole
        self.relationRolesBy = relationRolesBy  # 'name' or 'code'
        self.substringInAdminName = substringInAdminName
        self.catalogBindCode = catalogBindCode
        self.groupBindCode = groupBindCode
        self.setMinPriceIfDiscountRestrictMissing = setMinPriceIfDiscountRestrictMissing
        self.group2CatalogMapper = group2CatalogMapper


class SmagUkm4ConverterParameters(ConverterParameters):
    name = 'smag-ukm4'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, charset='866', format='dat', discount='modern',
                 defaultRoleCode=3, additionallyRoleCode=2, usersWithAdditionallyRole='', relationRolesBy='name',
                 substringInAdminName='Администратор', catalogBindCode='', groupBindCode='', group2CatalogMapper={}):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.charset = charset
        self.format = format
        self.discount = discount        # discount must be 'modern' or 'legacy'.
        self.defaultRoleCode = defaultRoleCode
        self.additionallyRoleCode = additionallyRoleCode
        self.usersWithAdditionallyRole = usersWithAdditionallyRole
        self.relationRolesBy = relationRolesBy  # 'name' or 'code'
        self.substringInAdminName = substringInAdminName
        self.catalogBindCode = catalogBindCode
        self.groupBindCode = groupBindCode
        self.group2CatalogMapper = group2CatalogMapper


class SmagXmlConverterParameters(ConverterParameters):
    name = 'smag-ukm-xml'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, mapTnvdCodeToTmcType=False, measureMapper={},
                 tnvdMapper={}):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.mapTnvdCodeToTmcType = mapTnvdCodeToTmcType
        self.measureMapper = measureMapper
        self.tnvdMapper = tnvdMapper


class ShtrihConverterParameters(ConverterParameters):
    name = 'shtrih'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, delimiter=None, cashierName=None,
                 hotKeyByBarcode=False, requireSaleRestrict=False):
        if not delimiter or delimiter == '':
            delimiter = ";"
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        p = re.compile('(\[)([0-9]+)(\])')
        m = p.match(delimiter)
        if m:
            delimiter = chr(int(m.group(2)))
        self.delimiter = delimiter
        if cashierName:
            self.cashierName = cashierName
        else:
            self.cashierName = False
        self.hotKeyByBarcode = hotKeyByBarcode
        self.requireSaleRestrict = requireSaleRestrict


class AtolConverterParameters(ConverterParameters):
    name = 'atol'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, useMultiplicityQuantityForMeasure=False, charset='cp1251', measureMapper={}):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.useMultiplicityQuantityForMeasure = useMultiplicityQuantityForMeasure
        self.charset = charset
        self.measureMapper = measureMapper


class UsersConverterParameters(ConverterParameters):
    name = 'users'

    def __init__(self, flags, data, errorDest=None, backup_dir=None):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)


class CrystalConverterParameters(ConverterParameters):
    name = 'crystal'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, visualVerifySG='', requireQuantityManualSG='', measureMapper=''):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        visualVerifySG = re.sub(r'\s', '', visualVerifySG)
        requireQuantityManualSG = re.sub(r'\s', '', requireQuantityManualSG)
        measureMapper = re.sub(r'\s', '', measureMapper)
        self.visualVerifySGs = visualVerifySG.split(',')
        self.requireQuantityManualSGs = requireQuantityManualSG.split(',')
        crystal_m = re.findall(r'([^-,]+)\-', measureMapper)
        aif_m = re.findall(r'\-([^-,]+)', measureMapper)
        self.measureMap = {}
        for i in range(0,len(crystal_m)):
            self.measureMap[crystal_m[i]] = aif_m[i]


class SapConverterParameters(ConverterParameters):
    name = 'sap'

    def __init__(self, flags, data, errorDest=None, backup_dir=None):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)


class Cif1ConverterParameters(ConverterParameters):
    name = 'cif1'

    def __init__(self, flags, data, errorDest=None, backup_dir=None):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)


class SimpleFileConverterParameters(ConverterParameters):
    name = 'simple_file'

    def __init__(self, flags, data, errorDest=None, backup_dir=None, cashPath=None, cashCodes=[]):
        ConverterParameters.__init__(self, flags, data, errorDest, backup_dir)
        self.cashPath = cashPath
        self.cashCodes = cashCodes


class Node(object):
    def __init__(self, id, backup_dir, converters, pack_size, inserter, badDictsDirectory, shopCode=None, extendedProperties={}, nesServer='127.0.0.1:28081'):
        self.id = id
        self.backup_dir = backup_dir
        self.converters = converters
        self.pack_size = pack_size
        self.inserter = inserter
        self.badDictsDirectory = badDictsDirectory
        self.shopCode = shopCode if shopCode else id
        self.extendedProperties = extendedProperties
        self.nesServer = nesServer

    def __str__(self):
        return 'id=%s shopCode=%s' % (self.id, self.shopCode)
